@@ -31,11 +31,13 @@
 #include <QGroupBox>
 #include <QGridLayout>
 #include <QSpinBox>
+#include <QClipboard>
 #include "plots.h"
 
 PlotView::PlotView(InputSource *input) : cursors(this), viewRange({0, 0})
 {
     mainSampleSource = input;
+    setDragMode(QGraphicsView::ScrollHandDrag);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     setMouseTracking(true);
     enableCursors(false);
@@ -94,16 +96,29 @@ void PlotView::contextMenuEvent(QContextMenuEvent * event)
         plotsMenu->addAction(action);
     }
 
-    // Add action to extract symbols from selected plot
-    auto extract = new QAction("Extract symbols (to stdout)...", &menu);
+    // Add submenu for extracting symbols
+    QMenu *extractMenu = menu.addMenu("Extract symbols");
+    // Add action to extract symbols from selected plot to stdout
+    auto extract = new QAction("To stdout", extractMenu);
     connect(
         extract, &QAction::triggered,
         this, [=]() {
-            extractSymbols(src);
+            extractSymbols(src, false);
         }
     );
     extract->setEnabled(cursorsEnabled && (src->sampleType() == typeid(float)));
-    menu.addAction(extract);
+    extractMenu->addAction(extract);
+
+    // Add action to extract symbols from selected plot to clipboard
+    auto extractClipboard = new QAction("Copy to clipboard", extractMenu);
+    connect(
+        extractClipboard, &QAction::triggered,
+        this, [=]() {
+            extractSymbols(src, true);
+        }
+    );
+    extractClipboard->setEnabled(cursorsEnabled && (src->sampleType() == typeid(float)));
+    extractMenu->addAction(extractClipboard);
 
     // Add action to export the selected samples into a file
     auto save = new QAction("Export samples to file...", &menu);
@@ -139,8 +154,8 @@ void PlotView::contextMenuEvent(QContextMenuEvent * event)
 void PlotView::cursorsMoved()
 {
     selectedSamples = {
-        horizontalScrollBar()->value() + cursors.selection().minimum * samplesPerLine(),
-        horizontalScrollBar()->value() + cursors.selection().maximum * samplesPerLine()
+        columnToSample(horizontalScrollBar()->value() + cursors.selection().minimum),
+        columnToSample(horizontalScrollBar()->value() + cursors.selection().maximum)
     };
 
     emitTimeSelection();
@@ -149,7 +164,7 @@ void PlotView::cursorsMoved()
 
 void PlotView::emitTimeSelection()
 {
-    off_t sampleCount = selectedSamples.length();
+    size_t sampleCount = selectedSamples.length();
     float selectionTime = sampleCount / (float)mainSampleSource->rate();
     emit timeSelectionChanged(selectionTime);
 }
@@ -170,18 +185,28 @@ bool PlotView::viewportEvent(QEvent *event) {
     if (event->type() == QEvent::Wheel) {
         QWheelEvent *wheelEvent = (QWheelEvent*)event;
         if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
-            if (wheelEvent->angleDelta().y() > 0) {
-                emit zoomIn();
-            } else if (wheelEvent->angleDelta().y() < 0) {
-                emit zoomOut();
+            bool canZoomIn = zoomLevel < fftSize;
+            bool canZoomOut = zoomLevel > 1;
+            int delta = wheelEvent->angleDelta().y();
+            if ((delta > 0 && canZoomIn) || (delta < 0 && canZoomOut)) {
+                scrollZoomStepsAccumulated += delta;
+
+                // `updateViewRange()` keeps the center sample in the same place after zoom. Apply
+                // a scroll adjustment to keep the sample under the mouse cursor in the same place instead.
+                zoomPos = wheelEvent->pos().x();
+                zoomSample = columnToSample(horizontalScrollBar()->value() + zoomPos);
+                if (scrollZoomStepsAccumulated >= 120) {
+                    scrollZoomStepsAccumulated -= 120;
+                    emit zoomIn();
+                } else if (scrollZoomStepsAccumulated <= -120) {
+                    scrollZoomStepsAccumulated += 120;
+                    emit zoomOut();
+                }
             }
             return true;
         }
     }
 
-    // Handle parent eveents
-    QAbstractScrollArea::viewportEvent(event);
-
     // Pass mouse events to individual plot objects
     if (event->type() == QEvent::MouseButtonPress ||
         event->type() == QEvent::MouseMove ||
@@ -211,10 +236,13 @@ bool PlotView::viewportEvent(QEvent *event) {
             if (cursors.mouseEvent(event->type(), *mouseEvent))
                 return true;
     }
-    return false;
+
+    // Handle parent eveents
+    return QGraphicsView::viewportEvent(event);
 }
 
-void PlotView::extractSymbols(std::shared_ptr<AbstractSampleSource> src)
+void PlotView::extractSymbols(std::shared_ptr<AbstractSampleSource> src,
+                              bool toClipboard)
 {
     if (!cursorsEnabled)
         return;
@@ -228,9 +256,18 @@ void PlotView::extractSymbols(std::shared_ptr<AbstractSampleSource> src)
     {
         symbols.push_back(samples[i]);
     }
-    for (auto f : symbols)
-        std::cout << f << ", ";
-    std::cout << std::endl << std::flush;
+    if (!toClipboard) {
+        for (auto f : symbols)
+            std::cout << f << ", ";
+        std::cout << std::endl << std::flush;
+    } else {
+        QClipboard *clipboard = QGuiApplication::clipboard();
+        QString symbolText;
+        QTextStream symbolStream(&symbolText);
+        for (auto f : symbols)
+            symbolStream << f << ", ";
+        clipboard->setText(symbolText);
+    }
 }
 
 void PlotView::exportSamples(std::shared_ptr<AbstractSampleSource> src)
@@ -282,6 +319,7 @@ void PlotView::exportSamples(std::shared_ptr<AbstractSampleSource> src)
 
     QGroupBox groupBox2("Decimation");
     QSpinBox decimation(&groupBox2);
+    decimation.setMinimum(1);
     decimation.setValue(1 / sampleSrc->relativeBandwidth());
 
     QVBoxLayout vbox2;
@@ -293,7 +331,7 @@ void PlotView::exportSamples(std::shared_ptr<AbstractSampleSource> src)
     if (dialog.exec()) {
         QStringList fileNames = dialog.selectedFiles();
        
-        off_t start, end;
+        size_t start, end;
         if (cursorSelection.isChecked()) {
             start = selectedSamples.minimum;
             end = start + selectedSamples.length();
@@ -307,12 +345,12 @@ void PlotView::exportSamples(std::shared_ptr<AbstractSampleSource> src)
 
         std::ofstream os (fileNames[0].toStdString(), std::ios::binary);
 
-        off_t index;
+        size_t index;
         // viewRange.length() is used as some less arbitrary step value
-        off_t step = viewRange.length();
+        size_t step = viewRange.length();
 
         for (index = start; index < end; index += step) {
-            off_t length = std::min(step, end - index); 
+            size_t length = std::min(step, end - index); 
             auto samples = sampleSrc->getSamples(index, length);
             if (samples != nullptr) {
                 for (auto i = 0; i < length; i += decimation.value()) {
@@ -326,7 +364,7 @@ void PlotView::exportSamples(std::shared_ptr<AbstractSampleSource> src)
 void PlotView::invalidateEvent()
 {
     horizontalScrollBar()->setMinimum(0);
-    horizontalScrollBar()->setMaximum(mainSampleSource->count());
+    horizontalScrollBar()->setMaximum(sampleToColumn(mainSampleSource->count()));
 }
 
 void PlotView::repaint()
@@ -360,8 +398,8 @@ void PlotView::setFFTAndZoom(int size, int zoom)
         spectrogramPlot->setZoomLevel(zoom);
 
     // Update horizontal (time) scrollbar
-    horizontalScrollBar()->setSingleStep(size * 10 / zoomLevel);
-    horizontalScrollBar()->setPageStep(size * 100 / zoomLevel);
+    horizontalScrollBar()->setSingleStep(10);
+    horizontalScrollBar()->setPageStep(100);
 
     updateView(true);
 }
@@ -415,7 +453,7 @@ void PlotView::paintEvent(QPaintEvent *event)
 #undef PLOT_LAYER
 }
 
-void PlotView::paintTimeScale(QPainter &painter, QRect &rect, range_t<off_t> sampleRange)
+void PlotView::paintTimeScale(QPainter &painter, QRect &rect, range_t<size_t> sampleRange)
 {
     float startTime = (float)sampleRange.minimum / sampleRate;
     float stopTime = (float)sampleRange.maximum / sampleRate;
@@ -441,8 +479,8 @@ void PlotView::paintTimeScale(QPainter &painter, QRect &rect, range_t<off_t> sam
 
     while (tick <= stopTime) {
 
-        off_t tickSample = tick * sampleRate;
-        int tickLine = (tickSample - sampleRange.minimum) / samplesPerLine();
+        size_t tickSample = tick * sampleRate;
+        int tickLine = sampleToColumn(tickSample - sampleRange.minimum);
 
         char buf[128];
         snprintf(buf, sizeof(buf), "%.06f", tick);
@@ -458,8 +496,8 @@ void PlotView::paintTimeScale(QPainter &painter, QRect &rect, range_t<off_t> sam
     tick = firstTick;
     while (tick <= stopTime) {
 
-        off_t tickSample = tick * sampleRate;
-        int tickLine = (tickSample - sampleRange.minimum) / samplesPerLine();
+        size_t tickSample = tick * sampleRate;
+        int tickLine = sampleToColumn(tickSample - sampleRange.minimum);
 
         painter.drawLine(tickLine, 0, tickLine, 10);
         tick += durationPerTick;
@@ -482,7 +520,7 @@ void PlotView::resizeEvent(QResizeEvent * event)
     updateView();
 }
 
-off_t PlotView::samplesPerLine()
+size_t PlotView::samplesPerColumn()
 {
     return fftSize / zoomLevel;
 }
@@ -494,34 +532,30 @@ void PlotView::scrollContentsBy(int dx, int dy)
 
 void PlotView::updateViewRange(bool reCenter)
 {
-    // Store old view for recentering
-    auto oldViewRange = viewRange;
-
     // Update current view
-    viewRange = {
-        horizontalScrollBar()->value(),
-        std::min(horizontalScrollBar()->value() + width() * samplesPerLine(), mainSampleSource->count())
-    };
+    auto start = columnToSample(horizontalScrollBar()->value());
+    viewRange = {start, std::min(start + columnToSample(width()), mainSampleSource->count())};
 
     // Adjust time offset to zoom around central sample
     if (reCenter) {
         horizontalScrollBar()->setValue(
-            horizontalScrollBar()->value() + (oldViewRange.length() - viewRange.length()) / 2
+            sampleToColumn(zoomSample) - zoomPos
         );
     }
+    // zoomSample = viewRange.minimum + viewRange.length() / 2;
+    // zoomPos = width() / 2;
 }
 
 void PlotView::updateView(bool reCenter)
 {
-    updateViewRange(reCenter);
-    horizontalScrollBar()->setMaximum(std::max(off_t(0), mainSampleSource->count() - ((width() - 1) * samplesPerLine())));
-
+    horizontalScrollBar()->setMaximum(std::max(0, sampleToColumn(mainSampleSource->count()) - width()));
     verticalScrollBar()->setMaximum(std::max(0, plotsHeight() - viewport()->height()));
+    updateViewRange(reCenter);
 
     // Update cursors
     range_t<int> newSelection = {
-        (int)((selectedSamples.minimum - horizontalScrollBar()->value()) / samplesPerLine()),
-        (int)((selectedSamples.maximum - horizontalScrollBar()->value()) / samplesPerLine())
+        sampleToColumn(selectedSamples.minimum) - horizontalScrollBar()->value(),
+        sampleToColumn(selectedSamples.maximum) - horizontalScrollBar()->value()
     };
     cursors.setSelection(newSelection);
 
@@ -529,7 +563,7 @@ void PlotView::updateView(bool reCenter)
     viewport()->update();
 }
 
-void PlotView::setSampleRate(off_t rate)
+void PlotView::setSampleRate(size_t rate)
 {
     sampleRate = rate;
 
@@ -549,3 +583,12 @@ void PlotView::enableScales(bool enabled)
     viewport()->update();
 }
 
+int PlotView::sampleToColumn(size_t sample)
+{
+    return sample / samplesPerColumn();
+}
+
+size_t PlotView::columnToSample(int col)
+{
+    return col * samplesPerColumn();
+}
